#include <math.h>
#include <stdio.h>
#include <stdlib.h>

#ifdef __APPLE__
#include <GLUT/glut.h>
#else
#include <GL/glut.h>
#endif

#include "matrix.h"
#include "psystem.h"

#define EPSILON 0.00001

Particle::Particle()
{
	alpha = 1.0;
	size = 1.0;
}


void Particle::set_start_position(const Vector3 &pos, double radius) {
	double rndx, rndy, rndz;
	double mag_square;

	if (radius < 1e-6) {
		this->pos = pos;
		return;
	}

	do {
		rndx = (2.0 * (double) rand() / RAND_MAX - 1.0) * radius;
		rndy = (2.0 * (double) rand() / RAND_MAX - 1.0) * radius;
		rndz = (2.0 * (double) rand() / RAND_MAX - 1.0) * radius;
		mag_square = rndx * rndx + rndy * rndy + rndz * rndz;
	} while (mag_square > radius * radius);

	this->pos = pos + Vector3(rndx, rndy, rndz);
	prev_pos = this->pos;
}

void Particle::set_start_velocity(const Vector3 &start_vel, double angle) {
	double len = length(start_vel);
	if(len < 1e-8) {
		v.x = v.y = v.z = 0.0;
		return;
	}

	Vector3 ivec, jvec, kvec;
	Vector3 xvec = Vector3(1.0, 0.0, 0.0);
	Vector3 zvec = Vector3(0.0, 0.0, 1.0);

	double xdot = dot(start_vel, xvec);
	double zdot = dot(start_vel, zvec);

	ivec = abs(xdot) < abs(zdot) ? xvec : zvec;

	jvec = start_vel / len;
	kvec = cross(ivec, jvec);
	ivec = cross(jvec, kvec);

	Matrix4x4 mat;
	mat.matrix[0][0] = ivec.x;
	mat.matrix[1][0] = ivec.y;
	mat.matrix[2][0] = ivec.z;

	mat.matrix[0][1] = jvec.x;
	mat.matrix[1][1] = jvec.y;
	mat.matrix[2][1] = jvec.z;

	mat.matrix[0][2] = kvec.x;
	mat.matrix[1][2] = kvec.y;
	mat.matrix[2][2] = kvec.z;

	double rnd1 = (double)rand() / RAND_MAX;
	double rnd2 = (double)rand() / RAND_MAX;

	double phi = rnd1 * angle;			//acos(pow(rnd1, 0.91 )); // * M_PI;
	double theta = 2.0 * M_PI * rnd2;

	v.x = cos(theta) * sin(phi) * len;
	v.y = cos(phi) * len;
	v.z = sin(theta) * sin(phi) * len;
	v.transform(mat);
}

void billboard(const Vector3 &v, const Color &col, double alpha, double sz)
{
	double m[16];
	double hsz = sz / 2.0;

	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();

	glTranslatef(v.x, v.y, v.z);

	glGetDoublev(GL_MODELVIEW_MATRIX, m);
	// make the upper 3x3 part of the matrix identity
	for(int i=0; i<10; i++) {
		m[i] = i % 5 ? 0.0 : 1.0;
	}
	glLoadMatrixd(m);

	glBegin(GL_QUADS);
	glColor4f(col.x, col.y, col.z, alpha);
	glTexCoord2f(0, 0);
	glVertex2f(-hsz, -hsz);
	glTexCoord2f(1, 0);
	glVertex2f(hsz, -hsz);
	glTexCoord2f(1, 1);
	glVertex2f(hsz, hsz);
	glTexCoord2f(0, 1);
	glVertex2f(-hsz, hsz);
	glEnd();

	glPopMatrix();
}
