#ifndef PSYSTEM_H_
#define PSYSTEM_H_

#include <list>
#include "particle.h"
#include "vector.h"

typedef Vector3 Color;

class PSystemParameters {
public:
	Vector3 f;								/* gravity vector = force */
	Vector3 start_vel;						/* particle initial velocity */
	double spawn_rate;						/* particles per second */
	double life;							/* lifetime of a particle */
	double damping;							/* damping factor */
	double spawn_radius;					/* radius of a sphere where the
											   particles' position can be */
	double spawn_cone;

	Color start_color, end_color;
	double start_alpha, end_alpha;

	double start_size, end_size;
	unsigned int tex;

	bool mblur;

	unsigned int src_blend, dst_blend;

	PSystemParameters();
};


class PSystem {
private:
	Vector3 pos;
	std::list<Particle*> particles;			/* particles */
	double time;							/* time since the particle system
											   started */
	double last_spawn;						/* time of the last spawn */

	PSystemParameters param;

public:
	PSystem();

	void set_position(const Vector3 &pos);

	void set_force(const Vector3 &f);
	void set_start_velocity(const Vector3 &vel);
	void set_spawn_rate(double pps);
	void set_lifetime(double life);
	void set_damping(double dfactor);
	void set_spawn_radius(double rad);
	void set_spawn_cone(double angle);

	void set_color(const Color &col);
	void set_color(const Color &c0, const Color &c1);

	void set_alpha(double alpha);
	void set_alpha(double a0, double a1);

	void set_size(double size);
	void set_size(double sz0, double sz1);

	void set_texture(unsigned int tex);
	bool set_texture(const char *fname);

	void set_motion_blur(bool enable);

	void set_blend_func(unsigned int s, unsigned int d);


	void update(double dt);
	void render();

	void spawn();
	void death();
	void simulate(double dt);
};

#endif
