#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <GL/gl.h>
#include "texture.h"

static unsigned char *load_ppm(const char *fname, int *xsz, int *ysz);

unsigned int load_texture(const char *fname, const char *alpha_fname)
{
	unsigned int tex, fmt;
	int xsz, ysz;
	unsigned char *rgb, *alpha, *pixels;

	if(!(rgb = load_ppm(fname, &xsz, &ysz))) {
		return 0;
	}

	if(alpha_fname) {
		int axsz, aysz;

		if(!(alpha = load_ppm(alpha_fname, &axsz, &aysz))) {
			delete [] rgb;
			return 0;
		}
		if(axsz != xsz || aysz != ysz) {
			fprintf(stderr, "texture %s size (%dx%d) doesn't match alpha texture %s (%dx%d)\n",
					fname, xsz, ysz, alpha_fname, axsz, aysz);
			delete [] rgb;
			delete [] alpha;
			return 0;
		}

		pixels = new unsigned char[xsz * ysz * 4];
		for(int i=0; i<xsz * ysz; i++) {
			pixels[i * 4] = rgb[i * 3];
			pixels[i * 4 + 1] = rgb[i * 3 + 1];
			pixels[i * 4 + 2] = rgb[i * 3 + 2];
			pixels[i * 4 + 3] = alpha[i * 3];
		}
		delete [] rgb;
		delete [] alpha;

		fmt = GL_RGBA;
	} else {
		pixels = rgb;
		fmt = GL_RGB;
	}


	glGenTextures(1, &tex);
	glBindTexture(GL_TEXTURE_2D, tex);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
	glTexImage2D(GL_TEXTURE_2D, 0, 4, xsz, ysz, 0, fmt, GL_UNSIGNED_BYTE, pixels);

	delete [] pixels;
	return tex;
}

static unsigned char *load_ppm(const char *fname, int *xsz, int *ysz)
{
	FILE *fp;
	int c, num_pix;
	size_t sz;
	unsigned char *pixels;

	if(!(fp = fopen(fname, "rb"))) {
		fprintf(stderr, "load_texture: failed to open: %s: %s\n", fname, strerror(errno));
		return 0;
	}
	if(fscanf(fp, "P6 %d %d 255", xsz, ysz) != 2) {
		fprintf(stderr, "load_texture: invalid format: %s\n", fname);
		return 0;
	}
	while((c = fgetc(fp)) != EOF) {
		if(c == '\n') break;
	}

	num_pix = *xsz * *ysz;
	sz = num_pix * 3;
	pixels = new unsigned char[num_pix * 3];
	if(fread(pixels, 1, sz, fp) < sz) {
		fprintf(stderr, "unexpected EOF while reading: %s\n", fname);
		return 0;
	}
	return pixels;
}
